"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var ts_smart_logger_1 = require("ts-smart-logger");
var util_1 = require("../../util");
var field_1 = require("../field");
var base_1 = require("../base");
var di_1 = require("../../di");
var AutoFocused = /** @class */ (function (_super) {
    __extends(AutoFocused, _super);
    /**
     * @stable [05.05.2018]
     * @param props
     */
    function AutoFocused(props) {
        var _this = _super.call(this, props) || this;
        _this.onDelay = _this.onDelay.bind(_this);
        _this.onKeyPress = _this.onKeyPress.bind(_this);
        _this.state = { focusedFieldValue: '' };
        _this.delayedTask = new util_1.DelayedTask(_this.onDelayedTask.bind(_this), props.delayTimeout, !props.useRobotMode);
        return _this;
    }
    /**
     * @stable [05.05.2018]
     */
    AutoFocused.prototype.componentDidMount = function () {
        _super.prototype.componentDidMount.call(this);
        if (this.props.useRobotMode) {
            this.eventManager.add(document, 'keypress', this.onKeyPress);
        }
        else {
            this.delayedTask.start();
        }
    };
    /**
     * @stable [05.05.2018]
     */
    AutoFocused.prototype.componentWillUnmount = function () {
        _super.prototype.componentWillUnmount.call(this);
        this.delayedTask.stop();
        if (this.props.useRobotMode) {
            this.eventManager.remove(document, 'keypress', this.onKeyPress);
        }
    };
    /**
     * @stable [04.05.2018]
     * @returns {JSX.Element}
     */
    AutoFocused.prototype.render = function () {
        var _this = this;
        var props = this.props;
        return (util_1.orNull(!props.useRobotMode, function () { return (React.createElement("div", { className: 'rac-invisible' },
            React.createElement(field_1.TextField, { ref: 'autoFocusedField', value: _this.state.focusedFieldValue, onChange: function (value) { return _this.setState({ focusedFieldValue: value }); }, plugins: [field_1.DelayedChangesFieldPlugin], delayTimeout: props.delayTimeout, onDelay: _this.onDelay }))); }));
    };
    /**
     * @stable [05.05.2018]
     * @param {IKeyboardEvent} e
     */
    AutoFocused.prototype.onKeyPress = function (e) {
        if (AutoFocused.alreadyFocused) {
            // Don't interfere with normal input mode
            return;
        }
        var props = this.props;
        var char = e.key;
        var charCode = e.keyCode;
        if (AutoFocused.ENTER_KEY_CODES.includes(charCode)) {
            if (props.ignoreEnterKeyCodeWrapper) {
                AutoFocused.logger.debug('[$AutoFocused][onKeyPress] Ignore enter key code and exit.');
                return;
            }
            char = '\n';
        }
        this.delayedTask.start();
        this.setState({ focusedFieldValue: this.state.focusedFieldValue + char });
    };
    /**
     * @stable [04.05.2018]
     */
    AutoFocused.prototype.onDelayedTask = function () {
        if (this.props.useRobotMode) {
            var focusedFieldValue = this.state.focusedFieldValue;
            if (focusedFieldValue.length >= AutoFocused.ROBOT_DETECTION_MIN_SYMBOLS_COUNT) {
                this.onDelay();
            }
            else {
                this.clearField();
            }
        }
        else {
            if (AutoFocused.alreadyFocused) {
                return;
            }
            this.autoFocusedField.setFocus();
        }
    };
    /**
     * @stable [04.05.2018]
     */
    AutoFocused.prototype.onDelay = function () {
        var onSelect = this.props.onSelect;
        if (onSelect) {
            onSelect(this.state.focusedFieldValue);
        }
        this.clearField();
    };
    /**
     * @stable [05.05.2018]
     */
    AutoFocused.prototype.clearField = function () {
        this.setState({ focusedFieldValue: field_1.FIELD_DISPLAY_EMPTY_VALUE });
    };
    Object.defineProperty(AutoFocused.prototype, "autoFocusedField", {
        /**
         * @stable [04.05.2018]
         * @returns {IField}
         */
        get: function () {
            return this.refs.autoFocusedField;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AutoFocused, "alreadyFocused", {
        /**
         * @stable [05.05.2018]
         * @returns {boolean}
         */
        get: function () {
            var activeElement = document.activeElement;
            return activeElement instanceof HTMLInputElement
                || activeElement instanceof HTMLTextAreaElement;
        },
        enumerable: true,
        configurable: true
    });
    AutoFocused.defaultProps = {
        delayTimeout: 300,
    };
    AutoFocused.logger = ts_smart_logger_1.LoggerFactory.makeLogger('AutoFocused');
    AutoFocused.ROBOT_DETECTION_MIN_SYMBOLS_COUNT = 3;
    AutoFocused.ENTER_KEY_CODES = [10, 13];
    __decorate([
        di_1.lazyInject(di_1.DI_TYPES.EventManager),
        __metadata("design:type", Object)
    ], AutoFocused.prototype, "eventManager", void 0);
    return AutoFocused;
}(base_1.BaseComponent));
exports.AutoFocused = AutoFocused;
//# sourceMappingURL=auto-focused.component.js.map