"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var R = require("ramda");
var ts_smart_logger_1 = require("ts-smart-logger");
var di_1 = require("../../di");
var definitions_interface_1 = require("../../definitions.interface");
var router_interface_1 = require("../../router/router.interface");
var form_action_builder_1 = require("../form/form-action.builder");
var util_1 = require("../../util");
var dictionary_1 = require("../../dictionary");
var UniversalContainer = /** @class */ (function (_super) {
    __extends(UniversalContainer, _super);
    /**
     * @stable - 12.04.2018
     * @param {TProps} props
     */
    function UniversalContainer(props) {
        var _this = _super.call(this, props) || this;
        // Because of Flux-architecture.
        // Needed for updating an array of dependent fields (each field depends on previous field state).
        // Each next field would be updated in one full cycle.
        _this.lifecycleTasks = [];
        _this.navigateToBack = _this.navigateToBack.bind(_this);
        _this.dispatchFormClear = _this.dispatchFormClear.bind(_this);
        var originalRenderer = _this.render;
        if (!R.isNil(originalRenderer)) {
            _this.render = function () {
                try {
                    return originalRenderer.call(_this);
                }
                catch (e) {
                    UniversalContainer.logger.error("[$UniversalContainer][constructor] The error:", e);
                    return _this.getErrorMessageElement(e);
                }
            };
        }
        return _this;
    }
    /**
     * @stable [02.07.2018]
     * @param {TProps} props
     * @param {Readonly<TState>} nextState
     * @returns {boolean}
     */
    UniversalContainer.prototype.shouldComponentUpdate = function (props, nextState) {
        return !R.equals(props, this.props)
            || (!R.isNil(this.state) && !R.equals(nextState, this.state));
    };
    /**
     * @stable [07.06.2018]
     * @param {Readonly<TProps extends IUniversalContainerProps>} prevProps
     * @param {Readonly<TState>} prevState
     */
    UniversalContainer.prototype.componentDidUpdate = function (prevProps, prevState) {
        var task = this.lifecycleTasks.pop();
        if (task) {
            task();
        }
    };
    /**
     * @stable [18.11.2018]
     * @param {string} type
     * @param {TChanges} data
     */
    UniversalContainer.prototype.dispatch = function (type, data) {
        var props = this.props;
        this.dispatchCustomType(props.sectionName + "." + type, util_1.applySection(props.sectionName, data));
    };
    /**
     * @stable [24.02.2019]
     * @param {string} type
     * @param {TData} data
     * @param {string} otherSection
     */
    UniversalContainer.prototype.dispatchFrameworkAction = function (type, data, otherSection) {
        var props = this.props;
        var section = otherSection || props.sectionName;
        this.dispatchCustomType("" + definitions_interface_1.ACTION_PREFIX + section + "." + type, util_1.applySection(section, data));
    };
    /**
     * @stable - 12.04.2018
     * @param {TPath0} path
     * @param {TState0} state
     */
    UniversalContainer.prototype.navigate = function (path, state) {
        this.doNavigate(router_interface_1.ROUTER_NAVIGATE_ACTION_TYPE, path, state);
    };
    /**
     * @stable [14.01.2019]
     * @param {TPath0} path
     * @param {TState0} state
     */
    UniversalContainer.prototype.navigateAndRewrite = function (path, state) {
        this.doNavigate(router_interface_1.ROUTER_REWRITE_ACTION_TYPE, path, state);
    };
    /**
     * @stable [24.02.2019]
     * @param {TChanges} changes
     * @param {string} otherSection
     */
    UniversalContainer.prototype.dispatchFormChanges = function (changes, otherSection) {
        this.appStore.dispatch(form_action_builder_1.FormActionBuilder.buildChangesSimpleAction(otherSection || this.props.sectionName, changes));
    };
    /**
     * @stable [24.02.2019]
     * @param {string} fieldName
     * @param {AnyT} fieldValue
     * @param {string} otherSection
     */
    UniversalContainer.prototype.dispatchFormChange = function (fieldName, fieldValue, otherSection) {
        this.appStore.dispatch(form_action_builder_1.FormActionBuilder.buildChangeSimpleAction(otherSection || this.props.sectionName, fieldName, fieldValue));
    };
    /**
     * @stable [18.11.2018]
     * @param {string} dictionary
     * @param {TData} data
     */
    UniversalContainer.prototype.dispatchLoadDictionary = function (dictionary, data) {
        this.dispatchCustomType(dictionary_1.DictionariesActionBuilder.buildLoadActionType(dictionary), util_1.applySection(dictionary, data));
    };
    /**
     * @stable [23.12.2018]
     * @param {string} type
     * @param {TData} data
     */
    UniversalContainer.prototype.dispatchCustomType = function (type, data) {
        this.appStore.dispatch({ type: type, data: data });
    };
    /**
     * @stable - 12.04.2018
     */
    UniversalContainer.prototype.navigateToBack = function () {
        this.dispatchCustomType(router_interface_1.ROUTER_BACK_ACTION_TYPE);
    };
    /**
     * @stable [27.12.2018]
     * @param {string | IOperationEntity} operation
     * @returns {boolean}
     */
    UniversalContainer.prototype.isTransportContainsExecutingOperation = function (operation) {
        return this.props.transport.queue.includes(util_1.isString(operation)
            ? operation
            : operation.id);
    };
    /**
     * @stable [26.08.2018]
     * @param {string} fieldName
     */
    UniversalContainer.prototype.dispatchFormClear = function (fieldName) {
        this.appStore.dispatch(form_action_builder_1.FormActionBuilder.buildClearSimpleAction(this.props.sectionName, fieldName));
    };
    Object.defineProperty(UniversalContainer.prototype, "appStore", {
        /**
         * @stable - 15.04.2018
         * @returns {Store<{}>}
         */
        get: function () {
            return di_1.staticInjector(di_1.DI_TYPES.Store);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniversalContainer.prototype, "dynamicRoutes", {
        /**
         * @stable - 15.04.2018
         * @returns {Map<IContainerClassEntity, IConnectorConfiguration>}
         */
        get: function () {
            return di_1.staticInjector(di_1.DI_TYPES.DynamicRoutes);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniversalContainer.prototype, "routes", {
        /**
         * @stable - 15.04.2018
         * @returns {IRoutesConfiguration}
         */
        get: function () {
            return di_1.staticInjector(di_1.DI_TYPES.Routes);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniversalContainer.prototype, "settings", {
        /**
         * @stable - 15.04.2018
         * @returns {ISettings}
         */
        get: function () {
            return di_1.staticInjector(di_1.DI_TYPES.Settings);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniversalContainer.prototype, "auth", {
        /**
         * @stable [26.04.2018]
         * @returns {IAuthService}
         */
        get: function () {
            return di_1.staticInjector(di_1.DI_TYPES.Auth);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniversalContainer.prototype, "t", {
        /**
         * @stable - 15.04.2018
         * @returns {TranslatorT}
         */
        get: function () {
            return di_1.staticInjector(di_1.DI_TYPES.Translate);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniversalContainer.prototype, "dc", {
        /**
         * @stable - 16.04.2018
         * @returns {IDateConverter}
         */
        get: function () {
            return di_1.staticInjector(di_1.DI_TYPES.DateConverter);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniversalContainer.prototype, "nc", {
        /**
         * @stable - 16.04.2018
         * @returns {INumberConverter}
         */
        get: function () {
            return di_1.staticInjector(di_1.DI_TYPES.NumberConverter);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniversalContainer.prototype, "uiFactory", {
        /**
         * @stable [18.05.2018]
         * @returns {IUIFactory}
         */
        get: function () {
            return di_1.staticInjector(di_1.DI_TYPES.UIFactory);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [07.06.2018]
     * @param {() => void} task
     */
    UniversalContainer.prototype.registerLifecycleTask = function (task) {
        this.lifecycleTasks.push(task);
    };
    /**
     * @stable [27.08.2018]
     * @param {Error} e
     * @returns {React.ReactNode}
     */
    UniversalContainer.prototype.getErrorMessageElement = function (e) {
        return util_1.buildErrorMessage(e.stack);
    };
    /**
     * @stable [14.01.2019]
     * @param {string} action
     * @param {TPath0} path
     * @param {TState0} state
     */
    UniversalContainer.prototype.doNavigate = function (action, path, state) {
        this.dispatchCustomType(action, util_1.toType({ path: path, state: state }));
    };
    UniversalContainer.logger = ts_smart_logger_1.LoggerFactory.makeLogger('UniversalContainer');
    return UniversalContainer;
}(React.Component));
exports.UniversalContainer = UniversalContainer;
//# sourceMappingURL=universal.container.js.map