"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var inversify_1 = require("inversify");
var URI = require("urijs");
var R = require("ramda");
var util_1 = require("../../../util");
var di_1 = require("../../../di");
var BaseTransportRequestPayloadFactory = /** @class */ (function () {
    function BaseTransportRequestPayloadFactory() {
    }
    /**
     * @stable [02.02.2019]
     * @param {ITransportRequestEntity} requestEntity
     * @param {ITransportCancelTokenEntity} cancelToken
     * @returns {ITransportRequestPayloadEntity}
     */
    BaseTransportRequestPayloadFactory.prototype.makeRequestPayload = function (requestEntity, cancelToken) {
        return util_1.defValuesFilter({
            url: this.getUrl(requestEntity),
            headers: this.getHeaders(requestEntity),
            method: this.getMethod(requestEntity),
            data: this.getData(requestEntity),
            cancelToken: cancelToken && cancelToken.token,
            withCredentials: this.getWithCredentials(requestEntity),
        });
    };
    /**
     * @stable [02.02.2019]
     * @param {ITransportRequestEntity} requestEntity
     * @returns {string}
     */
    BaseTransportRequestPayloadFactory.prototype.getUrl = function (requestEntity) {
        var transportSettings = this.settings.transport;
        var url = new URI(this.getBaseUrl(requestEntity));
        if (requestEntity.noCache !== true
            && !(R.isNil(requestEntity.method) || ['post'].includes(requestEntity.method))) {
            url.addSearch(transportSettings.noCachePrefix, Date.now());
        }
        return url.valueOf();
    };
    /**
     * @stable [02.02.2019]
     * @param {ITransportRequestEntity} requestEntity
     * @returns {string}
     */
    BaseTransportRequestPayloadFactory.prototype.getBaseUrl = function (requestEntity) {
        return requestEntity.url;
    };
    /**
     * @stable [02.02.2019]
     * @param {ITransportRequestEntity} requestEntity
     * @returns {boolean}
     */
    BaseTransportRequestPayloadFactory.prototype.getWithCredentials = function (requestEntity) {
        return requestEntity.withCredentials || this.settings.transport.withCredentials;
    };
    /**
     * @stable [02.02.2019]
     * @param {ITransportRequestEntity} requestEntity
     * @returns {string}
     */
    BaseTransportRequestPayloadFactory.prototype.getMethod = function (requestEntity) {
        return requestEntity.method || this.settings.transport.method;
    };
    /**
     * @stable [02.02.2019]
     * @param {ITransportRequestEntity} requestEntity
     * @returns {IKeyValue}
     */
    BaseTransportRequestPayloadFactory.prototype.getData = function (requestEntity) {
        return requestEntity.formData
            || requestEntity.blob
            || this.makeRequestPayloadData(requestEntity);
    };
    /**
     * @stable [02.02.2019]
     * @param {ITransportRequestEntity} requestEntity
     * @returns {IKeyValue}
     */
    BaseTransportRequestPayloadFactory.prototype.getHeaders = function (requestEntity) {
        var isBinaryData = this.isBinaryData(requestEntity);
        var isFormData = util_1.isDef(requestEntity.formData);
        var transportSettings = this.settings.transport;
        var result = __assign({}, requestEntity.headers);
        if (isBinaryData || isFormData) {
            return util_1.notNilValuesFilter(__assign({ 'Content-Type': isBinaryData
                    ? transportSettings.binaryContentType
                    : transportSettings.formDataContentType }, requestEntity.headers));
        }
        return util_1.notNilValuesFilter(result);
    };
    /**
     * @stable [02.02.2019]
     * @param {ITransportRequestEntity} requestEntity
     * @returns {boolean}
     */
    BaseTransportRequestPayloadFactory.prototype.isBinaryData = function (requestEntity) {
        return util_1.isDef(requestEntity.blob);
    };
    __decorate([
        di_1.lazyInject(di_1.DI_TYPES.Settings),
        __metadata("design:type", Object)
    ], BaseTransportRequestPayloadFactory.prototype, "settings", void 0);
    BaseTransportRequestPayloadFactory = __decorate([
        inversify_1.injectable()
    ], BaseTransportRequestPayloadFactory);
    return BaseTransportRequestPayloadFactory;
}());
exports.BaseTransportRequestPayloadFactory = BaseTransportRequestPayloadFactory;
//# sourceMappingURL=base-transport-request-payload-factory.service.js.map